library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
 
entity pet816cpu_tb is
end entity;
	 
architecture TB of pet816cpu_tb is
	 
    component PET816
       port( 
                        fastclkin : in STD_LOGIC;               -- 50MHz in

                        -- PET interface

                        phi0 : in STD_LOGIC;                    -- PET Phi1 in
                        rdy : in STD_LOGIC;
                        reset : in STD_LOGIC;

                        norom : out STD_LOGIC;          -- switch off PET ROM when set to '0'
                        rnw : out STD_LOGIC;

                        -- 65816 interface

                        cpuaddrin_64k : in STD_LOGIC_VECTOR (23 downto 16);
                        cpuaddrin_2k : in STD_LOGIC_VECTOR (15 downto 11);
                        cpuaddr_ismyio : in STD_LOGIC;          -- when 0 then addr bits 4 to 11 are all 1
                        cpuaddrin_16 : in STD_LOGIC_VECTOR (3 downto 0);
                        cpudata : inout STD_LOGIC_VECTOR (7 downto 0);

                        cpuvpa : in STD_LOGIC;
                        cpuvda : in STD_LOGIC;
                        cpuvp : in STD_LOGIC;
                        cpurnw : in STD_LOGIC;

                        cpuclk : out STD_LOGIC;

                        -- control

                        nslowbusen : out STD_LOGIC;                     -- when 0, pass CPU address bus to PET
                        nslowdataen : out STD_LOGIC;                    -- when 0, pass CPU data bus to PET
                        latchen: out STD_LOGIC;                                 -- '573 latch control (1=passthrough, 0=latch)

                        -- config
                        bootromin: in STD_LOGIC;                                -- when 1 then jumper is set to enable boot rom

                        -- fast memory interface (2 RAM modules)

                        nramcs : out STD_LOGIC_VECTOR (1 downto 0);

                        -- bootrom select, in-system-programming
                        nromcs : out STD_LOGIC;
                        nromwe : out STD_LOGIC;

                        -- external logic:
                        -- phi1 <= not(phi0);
                        -- phi2 <= not(phi1);
                        -- nramoe <= not(cpurnw);
                        -- cpuaddr_ismyio <= not( cpuaddrin(11 downto 4) = "1111111" );

                        diag : out STD_LOGIC

	 	);
    end component;

   component core_6502
     port (
	  clk           :in std_logic;
          reset         :in std_logic;
          irq_in        :in std_logic;
          nmi_in        :in std_logic;
          addr_pin      :out std_logic_vector (15 downto 0);
          din           :in  std_logic_vector (7 downto 0);
          dout          :out std_logic_vector (7 downto 0);
          dout_oe       :out std_logic;
          we_pin        :out std_logic;
          rd_pin        :out std_logic;
          sync          :out std_logic;
     	  vda_out	:out std_logic;
     	  vpa_out	:out std_logic
         );
  end component;

  component pet_rom_short 
    port (
	addr  :in std_logic_vector (14 downto 0);
        data  :out std_logic_vector (7 downto 0)
    );
  end component;

  component ram_dp
      generic (
      	       addr_bits          :integer;
               data_bits          :integer;
               register_out_flag  :integer := 0;
               block_type         :integer := 0
       );
       port (reset         :in  std_logic;
            wr_clk        :in  std_logic;
            wr_en     :in  std_logic;
            wr_addr       :in  std_logic_vector (addr_bits-1 downto 0);
            wr_data       :in  std_logic_vector(data_bits-1 downto 0);
            rd_clk        :in  std_logic;
            rd_addr       :in  std_logic_vector (addr_bits-1 downto 0);
            rd_data       :out std_logic_vector(data_bits-1 downto 0)
       );
    end component;

    signal reset:   		std_logic;
    signal clk:     		std_logic;
    signal petphi0:    		std_logic;
				
    signal petrdy:		std_logic := '1';
    signal petnorom:		std_logic;
    signal petrnw:		std_logic;

    signal irq:			std_logic :='1';
    signal nmi:			std_logic :='1';
       
    signal cpuaddr:		std_logic_vector(23 downto 0);
    signal nismyio:		std_logic := '1';
    signal cpudata:		std_logic_vector(7 downto 0);

    signal cpuvda:		std_logic := '1';
    signal cpuvpa:		std_logic := '1';
    signal cpuvp:		std_logic := '1';
    signal cpurnw:		std_logic := '1';
    signal cpuclk:		std_logic;

    signal nslowbusen:		std_logic;
    signal nslowdataen:		std_logic;
    signal latchen:		std_logic;

    signal bootromin:		std_logic := '0';

    signal nramcs:		std_logic_vector(1 downto 0);
    signal nromcs:		std_logic := '0';
    signal nromwe:		std_logic := '0';

    signal diag:		std_logic;

    signal core_clk:		std_logic;
    signal core_reset:		std_logic;
    signal core_irq:		std_logic;
    signal core_nmi:		std_logic;
    signal core_dout:		std_logic_vector(7 downto 0);
    signal core_dout_oe:	std_logic;
    signal core_wr:		std_logic;
    signal core_rd:		std_logic;
    signal core_sync:		std_logic;
    signal core_data_out:	std_logic_vector(7 downto 0);
    signal core_data_in:	std_logic_vector(7 downto 0);

    signal rom_data		:std_logic_vector(7 downto 0);

    signal slow_addr		:std_logic_vector(15 downto 0);
    signal slow_data_out	:std_logic_vector(7 downto 0);
    signal slow_data_in		:std_logic_vector(7 downto 0);

    signal ram_rd		:std_logic;
    signal ram_wr		:std_logic;
    signal ram_data_out		:std_logic_vector(7 downto 0);

begin

    ----------------------------------------------
    -- CPU core

    cpu: core_6502 port map (
    	core_clk,
	core_reset,
	core_irq,
	core_nmi,
	cpuaddr(15 downto 0),
	core_data_in,
	core_data_out,
	core_dout_oe,
	core_wr,
	core_rd,
	core_sync,
	cpuvda,
	cpuvpa
	);

    core_clk <= not(cpuclk);
    core_reset <= not(reset);
    core_irq <= not(irq);
    core_nmi <= not(nmi);
    cpurnw <= not(core_wr and not(core_rd));
    cpuaddr(23 downto 16) <= "00000000";

    DataOut: process(reset, core_data_out, cpudata, nslowdataen)
    begin
	if (reset='0') then
	    cpudata <= "ZZZZZZZZ";
	    slow_data_out <= "ZZZZZZZZ";
    	else 
	    cpudata <= core_data_out;

	    if (nslowdataen = '0') then
		slow_data_out <= cpudata;
	    else
		slow_data_out <= "ZZZZZZZZ";
	    end if;

    	end if;
    end process;

    AddrOut: process(reset, cpuaddr, nslowbusen)
    begin
	if (reset='0') then
	    slow_addr <= "ZZZZZZZZZZZZZZZZ";
    	else 
	    if (nslowbusen = '0') then
	   	slow_addr <= cpuaddr(15 downto 0);
	    else
		slow_addr <= "1111111111111111";
	    end if;
    	end if;
    end process;
	
    DataIn: process(reset, slow_data_in, cpudata, nslowdataen)
    begin
	if (reset='0') then
		core_data_in <= "ZZZZZZZZ";
	else
		if (nslowdataen = '0') then
			core_data_in <= slow_data_in;
		else
			core_data_in <= "ZZZZZZZZ";	-- TODO: fast memory
		end if;
	end if;
    end process;

    SlowDataIn: process(slow_addr,nslowdataen, rom_data, ram_data_out)
    begin
	if (nslowdataen = '1') then
		slow_data_in <= "ZZZZZZZZ";
	else
		if (slow_addr(15 downto 12) > "1000") then
			slow_data_in <= rom_data;
		else
			slow_data_in <= ram_data_out;
		end if;
	end if;
    end process;

    ----------------------------------------------
    -- ROM from $9000-$ffff

    rom: pet_rom_short port map (
	slow_addr(14 downto 0),
	rom_data
	);

    ----------------------------------------------
    -- RAM from $0000-$8fff

    ram: ram_dp 
    generic map (
    	15, 8, 0, 0
	)
    port map (
    	core_reset,
	ram_wr,
	'1',
	slow_addr(14 downto 0),
	slow_data_out,
	ram_rd,
	slow_addr(14 downto 0),
	ram_data_out
	);

    RamSel: process(slow_addr, petrnw, petphi0)
    begin
	    if (slow_addr(15 downto 12) <= "1000") then
		    ram_rd <= petrnw;
		    ram_wr <= not(petphi0) or petrnw;
	    else
		    ram_rd <= '0';
		    ram_wr <= '1';
	    end if;
    end process;


    ----------------------------------------------
    -- CPLD for the glue

    cpld: PET816 port map (
    		            clk,

                            -- PET interface

                            petphi0,
                            petrdy,
                            reset,

                            petnorom,
                            petrnw,

                            -- 65816 interface

                            cpuaddr(23 downto 16),
                            cpuaddr(15 downto 11),
			    nismyio,
                            cpuaddr(3 downto 0),

                            cpudata,

                            cpuvpa,
                            cpuvda,
                            cpuvp,
                            cpurnw,

                            cpuclk,

                            -- control

                            nslowbusen,
                            nslowdataen,
                            latchen,

                            -- config
                            bootromin,

                            -- fast memory interface (2 RAM modules)

                            nramcs,

                            -- bootrom select, in-system-programming
                            nromcs,
                            nromwe,

                            diag

    );
	 
    	process
    	begin
		clk <= '0';
		wait for 11 ns;
		clk <= '1';
		wait for 10 ns;
    	end process;

	process
	begin
		petphi0 <= '0';
		wait for 500 ns;
		petphi0 <= '1';
		wait for 500 ns;
	end process;

	process
	begin
		reset <= '0';
		wait for 155 ns;
		reset <= '1';
		wait for 10000000 ns;
	end process;

end;
